
module Pod

  # -------------------------------- find the target to exclude -------------------------------
  class FastProjectGenerating

    class << self
      attr_accessor :user_excluded_targets # Array<String>
      attr_accessor :excluded_target_names # Array<String>
    end

    self.user_excluded_targets = []
    self.excluded_target_names = []

    def self.should_skip_path(absolute_path)
      # may not support 'one pod for multiple platform'
      self.excluded_target_names.all? { |name| not (absolute_path.to_s.include? '/'+name+'/') }
    end
  end


  class PodTarget
    def uses_cpp?
      return @uses_cpp if defined? @uses_cpp
      @uses_cpp = begin
        file_accessors.reject { |a| a.spec.test_specification? }.any? do |file_accessor|
          file_accessor.source_files.any? { |sf| ['.cpp', '.mm', '.cc', '.cxx', '.c++', '.C'].include? sf.extname }
        end
      end
    end

    def uses_relative_header_importing?
      return @uses_relative_header_importing if defined?  @uses_relative_header_importing
      @uses_relative_header_importing = begin
        file_accessors.reject { |a| a.spec.test_specification? }.any? do |file_accessor|
          file_accessor.public_headers.any? do |file|
            file.extname == '.h' && begin
                File.read(file) =~ /#import +"[\w]+\/.+\.h"/
            end
          end
        end
      end
    end

  end

  class Installer

    OLD_FAST_INSTALL_2_generate_pods_project = instance_method :generate_pods_project
    define_method :generate_pods_project do |*args|

      # check the input names
      names = pod_targets.map(&:name)
      wrong_input_names = FastProjectGenerating.user_excluded_targets.select { |input| not (names.include? input) }
      unless wrong_input_names.empty?
        puts "Wrong input for accelerate_xcodeproj_generating! #{wrong_input_names}".red
        FastProjectGenerating.user_excluded_targets -= wrong_input_names
      end

      # find the pod that cannot support
      relative_importing_target_names = []
      cpp_target_names = []

      pod_targets.map do |target|
        if target.uses_cpp?
          cpp_target_names << target.name
        elsif target.uses_relative_header_importing?
          relative_importing_target_names << target.name
        end
      end

      FastProjectGenerating.excluded_target_names = []
      FastProjectGenerating.excluded_target_names += FastProjectGenerating.user_excluded_targets
      FastProjectGenerating.excluded_target_names += relative_importing_target_names


      OLD_FAST_INSTALL_2_generate_pods_project.bind(self).(*args)


      unless FastProjectGenerating.excluded_target_names.empty?
        puts "Skip excluding headers (manually specified): #{FastProjectGenerating.user_excluded_targets}" unless FastProjectGenerating.user_excluded_targets.empty?
        puts "Skip excluding headers (contain c++): #{cpp_target_names}" unless cpp_target_names.empty?
        puts "Skip excluding headers (contain relative importing): #{relative_importing_target_names}" unless relative_importing_target_names.empty?
      end

    end

  end


  #---------------------------------- real function ------------------------------------


  # Filter the Headers when adding file references (the origin)
  class Project < Xcodeproj::Project

    FAST_INSTALL_OLD_add_file_reference = instance_method :add_file_reference
    define_method(:add_file_reference) do |absolute_path, group, reflect_file_system_structure, base_path = nil|
      if absolute_path.extname == '.h'
        if FastProjectGenerating.should_skip_path(absolute_path)
          next
        end
      end

      FAST_INSTALL_OLD_add_file_reference.bind(self).(absolute_path, group, false, base_path)
    end

  end


  # Filter the Headers when adding to build phase
  class Installer
    class Xcode
      class PodsProjectGenerator
        class PodTargetInstaller

          # 其实应该直接修改 def add_files_to_build_phases(native_target, test_native_targets)
          # 但 hook 这个函数更方便
          FAST_INSTALL_OLD_project_file_references_array = instance_method :project_file_references_array
          define_method(:project_file_references_array) do |files, file_type|

            if file_type == 'header'
              files = files.reject do |path|
                FastProjectGenerating.should_skip_path(path)
              end
            end
            next FAST_INSTALL_OLD_project_file_references_array.bind(self).(files, file_type)

          end

        end
      end
    end
  end


  # There's a bug when use `header_dir` in podspec.
  #
  # The `header search path` is set to something like
  #  "${PODS_ROOT}/Headers/Private/lottie-ios"
  #
  # But the when header_dir is set, the real headers will place in a nested folder. So we add a recursise search.
  #
  # Xcode cannot find the header file when it isn't added to the project AND the header file doesn't
  # placed in the same directory with the source file who import it if header search path is not set.
  #
  #
  class PodTarget


    FAST_INSTALL_OLD_header_search_paths = instance_method :header_search_paths
    define_method :header_search_paths do |*args|
      paths = FAST_INSTALL_OLD_header_search_paths.bind(self).(*args)
      paths = paths.map do |path|
        if path.end_with?('Public') || path.end_with?('Private')
          path
        else
          path + '/**' # recursive
        end
      end
      paths
    end

  end


  # class Sandbox
  #   class HeadersStore
  #
  #     def add_file(namespace, relative_header_path, mkdir: true)
  #       namespaced_path = root
  #       namespaced_path.mkpath if mkdir
  #
  #       absolute_source = (sandbox.root + relative_header_path)
  #       namespaced_path += absolute_source.relative_path_from(sandbox.root).dirname
  #       namespaced_path.mkpath unless namespaced_path.exist?
  #
  #       source = absolute_source.relative_path_from(namespaced_path)
  #       FileUtils.ln_sf(source, namespaced_path)
  #       namespaced_path + relative_header_path.basename
  #     end
  #
  #   end
  # end



end


